@@ -162,7 +162,7 @@ class Resource(RequestIdMixin):
         return None
 
     def _add_details(self, info):
-        for (k, v) in six.iteritems(info):
+        for (k, v) in info.items():
             try:
                 setattr(self, k, v)
                 self._info[k] = v
@@ -210,7 +210,9 @@ class Resource(RequestIdMixin):
         return self._info == other._info
 
     def __ne__(self, other):
-        return not self.__eq__(other)
+        # Using not of '==' implementation because the not of
+        # __eq__, when it returns NotImplemented, is returning False.
+        return not self == other
 
     def is_loaded(self):
         return self._loaded
@@ -218,6 +220,9 @@ class Resource(RequestIdMixin):
     def set_loaded(self, val):
         self._loaded = val
 
+    def set_info(self, key, value):
+        self._info[key] = value
+
     def to_dict(self):
         return copy.deepcopy(self._info)
 
@@ -488,7 +493,7 @@ class BootingManagerWithFind(ManagerWithFind):
 
         bdm = []
 
-        for device_name, mapping in six.iteritems(block_device_mapping):
+        for device_name, mapping in block_device_mapping.items():
             #
             # The mapping is in the format:
             # <id>:[<type>]:[<size(GB)>]:[<delete_on_terminate>]
