@@ -12,44 +12,68 @@
 
 import os
 import time
-import uuid
 
 from cinderclient.v2 import client as cinderclient
 import fixtures
-from keystoneauth1 import loading
+from glanceclient import client as glanceclient
+from keystoneauth1.exceptions import discovery as discovery_exc
+from keystoneauth1 import identity
 from keystoneauth1 import session as ksession
 from keystoneclient import client as keystoneclient
+from keystoneclient import discover as keystone_discover
+from neutronclient.v2_0 import client as neutronclient
 import os_client_config
-import six
+from oslo_utils import uuidutils
 import tempest.lib.cli.base
 import testtools
 
 import novaclient
 import novaclient.api_versions
 import novaclient.client
+from novaclient.v2 import networks
 import novaclient.v2.shell
 
 BOOT_IS_COMPLETE = ("login as 'cirros' user. default password: "
                     "'cubswin:)'. use 'sudo' for root.")
 
 
+def is_keystone_version_available(session, version):
+    """Given a (major, minor) pair, check if the API version is enabled."""
+
+    d = keystone_discover.Discover(session)
+    try:
+        d.create_client(version)
+    except (discovery_exc.DiscoveryFailure, discovery_exc.VersionNotAvailable):
+        return False
+    else:
+        return True
+
+
 # The following are simple filter functions that filter our available
 # image / flavor list so that they can be used in standard testing.
 def pick_flavor(flavors):
     """Given a flavor list pick a reasonable one."""
-    for flavor in flavors:
-        if flavor.name == 'm1.tiny':
-            return flavor
-    for flavor in flavors:
-        if flavor.name == 'm1.small':
-            return flavor
+    for flavor_priority in ('m1.nano', 'm1.micro', 'm1.tiny', 'm1.small'):
+        for flavor in flavors:
+            if flavor.name == flavor_priority:
+                return flavor
     raise NoFlavorException()
 
 
 def pick_image(images):
+    firstImage = None
     for image in images:
-        if image.name.startswith('cirros') and image.name.endswith('-uec'):
+        firstImage = firstImage or image
+        if image.name.startswith('cirros') and (
+                image.name.endswith('-uec') or
+                image.name.endswith('-disk.img')):
             return image
+
+    # We didn't find the specific cirros image we'd like to use, so just use
+    # the first available.
+    if firstImage:
+        return firstImage
+
     raise NoImageException()
 
 
@@ -57,7 +81,7 @@ def pick_network(networks):
     network_name = os.environ.get('OS_NOVACLIENT_NETWORK')
     if network_name:
         for network in networks:
-            if network.label == network_name:
+            if network.name == network_name:
                 return network
         raise NoNetworkException()
     return networks[0]
@@ -83,6 +107,9 @@ class NoCloudConfigException(Exception):
     pass
 
 
+CACHE = {}
+
+
 class ClientTestBase(testtools.TestCase):
     """Base test class for read only python-novaclient commands.
 
@@ -168,32 +195,53 @@ class ClientTestBase(testtools.TestCase):
 
         user = auth_info['username']
         passwd = auth_info['password']
-        tenant = auth_info['project_name']
+        self.project_name = auth_info['project_name']
         auth_url = auth_info['auth_url']
+        user_domain_id = auth_info['user_domain_id']
         self.project_domain_id = auth_info['project_domain_id']
+
         if 'insecure' in cloud_config.config:
             self.insecure = cloud_config.config['insecure']
         else:
             self.insecure = False
 
-        if self.COMPUTE_API_VERSION == "2.latest":
-            version = novaclient.API_MAX_VERSION.get_string()
-        else:
-            version = self.COMPUTE_API_VERSION or "2"
-
-        loader = loading.get_plugin_loader("password")
-        auth = loader.load_from_options(username=user,
-                                        password=passwd,
-                                        project_name=tenant,
-                                        auth_url=auth_url)
+        auth = identity.Password(username=user,
+                                 password=passwd,
+                                 project_name=self.project_name,
+                                 auth_url=auth_url,
+                                 project_domain_id=self.project_domain_id,
+                                 user_domain_id=user_domain_id)
         session = ksession.Session(auth=auth, verify=(not self.insecure))
 
-        self.client = novaclient.client.Client(version, session=session)
+        self.client = self._get_novaclient(session)
+
+        self.glance = glanceclient.Client('2', session=session)
 
         # pick some reasonable flavor / image combo
-        self.flavor = pick_flavor(self.client.flavors.list())
-        self.image = pick_image(self.client.images.list())
-        self.network = pick_network(self.client.networks.list())
+        if "flavor" not in CACHE:
+            CACHE["flavor"] = pick_flavor(self.client.flavors.list())
+        if "image" not in CACHE:
+            CACHE["image"] = pick_image(self.glance.images.list())
+        self.flavor = CACHE["flavor"]
+        self.image = CACHE["image"]
+
+        if "network" not in CACHE:
+            # Get the networks from neutron.
+            neutron = neutronclient.Client(session=session)
+            neutron_networks = neutron.list_networks()['networks']
+            # Convert the neutron dicts to Network objects.
+            nets = []
+            for network in neutron_networks:
+                nets.append(networks.Network(
+                    networks.NeutronManager, network))
+            # Keep track of whether or not there are multiple networks
+            # available to the given tenant because if so, a specific
+            # network ID has to be passed in on server create requests
+            # otherwise the server POST will fail with a 409.
+            CACHE['multiple_networks'] = len(nets) > 1
+            CACHE["network"] = pick_network(nets)
+        self.network = CACHE["network"]
+        self.multiple_networks = CACHE['multiple_networks']
 
         # create a CLI client in case we'd like to do CLI
         # testing. tempest.lib does this really weird thing where it
@@ -207,7 +255,7 @@ class ClientTestBase(testtools.TestCase):
         self.cli_clients = tempest.lib.cli.base.CLIClient(
             username=user,
             password=passwd,
-            tenant_name=tenant,
+            tenant_name=self.project_name,
             uri=auth_url,
             cli_dir=cli_dir,
             insecure=self.insecure)
@@ -217,6 +265,47 @@ class ClientTestBase(testtools.TestCase):
                                               password=passwd)
         self.cinder = cinderclient.Client(auth=auth, session=session)
 
+    def _get_novaclient(self, session):
+        nc = novaclient.client.Client("2", session=session)
+
+        if self.COMPUTE_API_VERSION:
+            if "min_api_version" not in CACHE:
+                # Obtain supported versions by API side
+                v = nc.versions.get_current()
+                if not hasattr(v, 'version') or not v.version:
+                    # API doesn't support microversions
+                    CACHE["min_api_version"] = (
+                        novaclient.api_versions.APIVersion("2.0"))
+                    CACHE["max_api_version"] = (
+                        novaclient.api_versions.APIVersion("2.0"))
+                else:
+                    CACHE["min_api_version"] = (
+                        novaclient.api_versions.APIVersion(v.min_version))
+                    CACHE["max_api_version"] = (
+                        novaclient.api_versions.APIVersion(v.version))
+
+            if self.COMPUTE_API_VERSION == "2.latest":
+                requested_version = min(novaclient.API_MAX_VERSION,
+                                        CACHE["max_api_version"])
+            else:
+                requested_version = novaclient.api_versions.APIVersion(
+                    self.COMPUTE_API_VERSION)
+
+            if not requested_version.matches(CACHE["min_api_version"],
+                                             CACHE["max_api_version"]):
+                msg = ("%s is not supported by Nova-API. Supported version" %
+                       self.COMPUTE_API_VERSION)
+                if CACHE["min_api_version"] == CACHE["max_api_version"]:
+                    msg += ": %s" % CACHE["min_api_version"].get_string()
+                else:
+                    msg += "s: %s - %s" % (
+                        CACHE["min_api_version"].get_string(),
+                        CACHE["max_api_version"].get_string())
+                self.skipTest(msg)
+
+            nc.api_version = requested_version
+        return nc
+
     def nova(self, action, flags='', params='', fail_ok=False,
              endpoint_type='publicURL', merge_stderr=False):
         if self.COMPUTE_API_VERSION:
@@ -243,7 +332,7 @@ class ClientTestBase(testtools.TestCase):
             self.fail("Volume %s did not reach status %s after %d s"
                       % (volume.id, status, timeout))
 
-    def wait_for_server_os_boot(self, server_id, timeout=60,
+    def wait_for_server_os_boot(self, server_id, timeout=300,
                                 poll_interval=1):
         """Wait until instance's operating system  is completely booted.
 
@@ -252,13 +341,15 @@ class ClientTestBase(testtools.TestCase):
         :param poll_interval: poll interval in seconds
         """
         start_time = time.time()
+        console = None
         while time.time() - start_time < timeout:
-            if BOOT_IS_COMPLETE in self.nova('console-log %s ' % server_id):
+            console = self.nova('console-log %s ' % server_id)
+            if BOOT_IS_COMPLETE in console:
                 break
             time.sleep(poll_interval)
         else:
-            self.fail("Server %s did not boot after %d s"
-                      % (server_id, timeout))
+            self.fail("Server %s did not boot after %d s.\nConsole:\n%s"
+                      % (server_id, timeout, console))
 
     def wait_for_resource_delete(self, resource, manager,
                                  timeout=60, poll_interval=1):
@@ -287,7 +378,7 @@ class ClientTestBase(testtools.TestCase):
 
         :param prefix: string prefix
         """
-        name = "%s-%s" % (prefix, six.text_type(uuid.uuid4()))
+        name = "%s-%s" % (prefix, uuidutils.generate_uuid())
         return name
 
     def _get_value_from_the_table(self, table, key):
@@ -309,7 +400,8 @@ class ClientTestBase(testtools.TestCase):
                 l_property, l_value = line.split("|")[1:3]
                 if l_property.strip() == key:
                     return l_value.strip()
-        raise ValueError("Property '%s' is missing from the table." % key)
+        raise ValueError("Property '%s' is missing from the table:\n%s" %
+                         (key, table))
 
     def _get_column_value_from_single_row_table(self, table, column):
         """Get the value for the column in the single-row table
@@ -392,6 +484,11 @@ class ClientTestBase(testtools.TestCase):
             'building', ['active'])
         return server
 
+    def _wait_for_state_change(self, server_id, status):
+        novaclient.v2.shell._poll_for_status(
+            self.client.servers.get, server_id, None, [status],
+            show_progress=False, poll_period=1, silent=True)
+
     def _get_project_id(self, name):
         """Obtain project id by project name."""
         if self.keystone.version == "v3":
@@ -400,6 +497,21 @@ class ClientTestBase(testtools.TestCase):
             project = self.keystone.tenants.find(name=name)
         return project.id
 
+    def _cleanup_server(self, server_id):
+        """Deletes a server and waits for it to be gone."""
+        self.client.servers.delete(server_id)
+        self.wait_for_resource_delete(server_id, self.client.servers)
+
+    def _get_absolute_limits(self):
+        """Returns the absolute limits (quota usage) including reserved quota
+        usage for the given tenant running the test.
+
+        :return: A dict where the key is the limit (or usage) and value.
+        """
+        # The absolute limits are returned in a generator so convert to a dict.
+        return {limit.name: limit.value
+                for limit in self.client.limits.get(reserved=True).absolute}
+
 
 class TenantTestBase(ClientTestBase):
     """Base test class for additional tenant and user creation which
