@@ -1457,3 +1457,73 @@ class ServersCreateImageBackupV2_45Test(utils.FixturedTestCase):
         self.assertEqual('456', sb['image_id'])
         self.assert_request_id(sb, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/action')
+
+
+class ServersV249Test(ServersV2_37Test):
+
+    api_version = "2.49"
+
+    def test_interface_attach_with_tag(self):
+        s = self.cs.servers.get(1234)
+        ret = s.interface_attach('7f42712e-63fe-484c-a6df-30ae4867ff66',
+                                 None, None, 'test_tag')
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called(
+            'POST', '/servers/1234/os-interface',
+            {'interfaceAttachment':
+                {'port_id': '7f42712e-63fe-484c-a6df-30ae4867ff66',
+                 'tag': 'test_tag'}})
+
+    def test_add_fixed_ip(self):
+        # novaclient.v2.servers.Server.add_fixed_ip()
+        # is not available after 2.44
+        pass
+
+    def test_remove_fixed_ip(self):
+        # novaclient.v2.servers.Server.remove_fixed_ip()
+        # is not available after 2.44
+        pass
+
+
+class ServersV252Test(ServersV249Test):
+
+    api_version = "2.52"
+
+    def test_create_server_with_tags(self):
+        self.cs.servers.create(
+            name="My server",
+            image=1,
+            flavor=1,
+            meta={'foo': 'bar'},
+            userdata="hello moto",
+            key_name="fakekey",
+            nics=self._get_server_create_default_nics(),
+            tags=['tag1', 'tag2']
+        )
+        self.assert_called('POST', '/servers',
+                           {'server': {
+                               'flavorRef': '1',
+                               'imageRef': '1',
+                               'key_name': 'fakekey',
+                               'max_count': 1,
+                               'metadata': {'foo': 'bar'},
+                               'min_count': 1,
+                               'name': 'My server',
+                               'networks': 'auto',
+                               'tags': ['tag1', 'tag2'],
+                               'user_data': 'aGVsbG8gbW90bw=='
+                           }}
+                           )
+
+    def test_create_server_with_tags_pre_252_fails(self):
+        self.cs.api_version = api_versions.APIVersion('2.51')
+        self.assertRaises(exceptions.UnsupportedAttribute,
+                          self.cs.servers.create,
+                          name="My server",
+                          image=1,
+                          flavor=1,
+                          meta={'foo': 'bar'},
+                          userdata="hello moto",
+                          key_name="fakekey",
+                          nics=self._get_server_create_default_nics(),
+                          tags=['tag1', 'tag2'])
