@@ -20,6 +20,7 @@ Server interface.
 """
 
 import base64
+import warnings
 
 from oslo_utils import encodeutils
 import six
@@ -30,11 +31,33 @@ from novaclient import base
 from novaclient import crypto
 from novaclient import exceptions
 from novaclient.i18n import _
-from novaclient.v2 import security_groups
 
 
 REBOOT_SOFT, REBOOT_HARD = 'SOFT', 'HARD'
 
+CONSOLE_TYPE_ACTION_MAPPING = {
+    'novnc': 'os-getVNCConsole',
+    'xvpvnc': 'os-getVNCConsole',
+    'spice-html5': 'os-getSPICEConsole',
+    'rdp-html5': 'os-getRDPConsole',
+    'serial': 'os-getSerialConsole'
+}
+
+CONSOLE_TYPE_PROTOCOL_MAPPING = {
+    'novnc': 'vnc',
+    'xvpvnc': 'vnc',
+    'spice-html5': 'spice',
+    'rdp-html5': 'rdp',
+    'serial': 'serial',
+    'webmks': 'mks'
+}
+
+ADD_REMOVE_FIXED_FLOATING_DEPRECATION_WARNING = _(
+    'The %s server action API is deprecated as of the 2.44 microversion. This '
+    'API binding will be removed in the first major release after the Nova '
+    '16.0.0 Pike release. Use python-neutronclient or openstacksdk instead.'
+)
+
 
 class Server(base.Resource):
     HUMAN_ID = True
@@ -121,6 +144,15 @@ class Server(base.Resource):
         """
         return self.manager.get_mks_console(self)
 
+    def get_console_url(self, console_type):
+        """
+        Retrieve a console of a particular protocol and console_type
+
+        :param console_type: Type of console
+        """
+
+        return self.manager.get_console_url(self, console_type)
+
     def get_password(self, private_key=None):
         """
         Get password for a Server.
@@ -545,12 +577,21 @@ class Server(base.Resource):
         """
         return self.manager.interface_list(self)
 
+    @api_versions.wraps("2.0", "2.48")
     def interface_attach(self, port_id, net_id, fixed_ip):
         """
         Attach a network interface to an instance.
         """
         return self.manager.interface_attach(self, port_id, net_id, fixed_ip)
 
+    @api_versions.wraps("2.49")
+    def interface_attach(self, port_id, net_id, fixed_ip, tag=None):
+        """
+        Attach a network interface to an instance with an optional tag.
+        """
+        return self.manager.interface_attach(self, port_id, net_id, fixed_ip,
+                                             tag)
+
     def interface_detach(self, port_id):
         """
         Detach a network interface from an instance.
@@ -606,6 +647,11 @@ class NetworkInterface(base.Resource):
         return '<NetworkInterface: %s>' % self.id
 
 
+class SecurityGroup(base.Resource):
+    def __str__(self):
+        return str(self.id)
+
+
 class ServerManager(base.BootingManagerWithFind):
     resource_class = Server
 
@@ -617,7 +663,7 @@ class ServerManager(base.BootingManagerWithFind):
               block_device_mapping_v2=None, nics=None, scheduler_hints=None,
               config_drive=None, admin_pass=None, disk_config=None,
               access_ip_v4=None, access_ip_v6=None, description=None,
-              **kwargs):
+              tags=None, **kwargs):
         """
         Create (boot) a new server.
         """
@@ -709,27 +755,32 @@ class ServerManager(base.BootingManagerWithFind):
             body['server']['block_device_mapping_v2'] = block_device_mapping_v2
 
         if nics is not None:
-            # NOTE(tr3buchet): nics can be an empty list
-            all_net_data = []
-            for nic_info in nics:
-                net_data = {}
-                # if value is empty string, do not send value in body
-                if nic_info.get('net-id'):
-                    net_data['uuid'] = nic_info['net-id']
-                if (nic_info.get('v4-fixed-ip') and
-                        nic_info.get('v6-fixed-ip')):
-                    raise base.exceptions.CommandError(_(
-                        "Only one of 'v4-fixed-ip' and 'v6-fixed-ip' may be"
-                        " provided."))
-                elif nic_info.get('v4-fixed-ip'):
-                    net_data['fixed_ip'] = nic_info['v4-fixed-ip']
-                elif nic_info.get('v6-fixed-ip'):
-                    net_data['fixed_ip'] = nic_info['v6-fixed-ip']
-                if nic_info.get('port-id'):
-                    net_data['port'] = nic_info['port-id']
-                if nic_info.get('tag'):
-                    net_data['tag'] = nic_info['tag']
-                all_net_data.append(net_data)
+            # With microversion 2.37+ nics can be an enum of 'auto' or 'none'
+            # or a list of dicts.
+            if isinstance(nics, six.string_types):
+                all_net_data = nics
+            else:
+                # NOTE(tr3buchet): nics can be an empty list
+                all_net_data = []
+                for nic_info in nics:
+                    net_data = {}
+                    # if value is empty string, do not send value in body
+                    if nic_info.get('net-id'):
+                        net_data['uuid'] = nic_info['net-id']
+                    if (nic_info.get('v4-fixed-ip') and
+                            nic_info.get('v6-fixed-ip')):
+                        raise base.exceptions.CommandError(_(
+                            "Only one of 'v4-fixed-ip' and 'v6-fixed-ip' "
+                            "may be provided."))
+                    elif nic_info.get('v4-fixed-ip'):
+                        net_data['fixed_ip'] = nic_info['v4-fixed-ip']
+                    elif nic_info.get('v6-fixed-ip'):
+                        net_data['fixed_ip'] = nic_info['v6-fixed-ip']
+                    if nic_info.get('port-id'):
+                        net_data['port'] = nic_info['port-id']
+                    if nic_info.get('tag'):
+                        net_data['tag'] = nic_info['tag']
+                    all_net_data.append(net_data)
             body['server']['networks'] = all_net_data
 
         if disk_config is not None:
@@ -744,6 +795,9 @@ class ServerManager(base.BootingManagerWithFind):
         if description:
             body['server']['description'] = description
 
+        if tags:
+            body['server']['tags'] = tags
+
         return self._create(resource_url, body, response_key,
                             return_raw=return_raw, **kwargs)
 
@@ -790,7 +844,7 @@ class ServerManager(base.BootingManagerWithFind):
 
         qparams = {}
 
-        for opt, val in six.iteritems(search_opts):
+        for opt, val in search_opts.items():
             if val:
                 if isinstance(val, six.text_type):
                     val = val.encode('utf-8')
@@ -836,29 +890,36 @@ class ServerManager(base.BootingManagerWithFind):
             marker = result[-1].id
         return result
 
+    @api_versions.wraps('2.0', '2.43')
     def add_fixed_ip(self, server, network_id):
         """
-        Add an IP address on a network.
+        DEPRECATED Add an IP address on a network.
 
         :param server: The :class:`Server` (or its ID) to add an IP to.
         :param network_id: The ID of the network the IP should be on.
         :returns: An instance of novaclient.base.TupleWithMeta
         """
+        warnings.warn(ADD_REMOVE_FIXED_FLOATING_DEPRECATION_WARNING %
+                      'addFixedIP', DeprecationWarning)
         return self._action('addFixedIp', server, {'networkId': network_id})
 
+    @api_versions.wraps('2.0', '2.43')
     def remove_fixed_ip(self, server, address):
         """
-        Remove an IP address.
+        DEPRECATED Remove an IP address.
 
         :param server: The :class:`Server` (or its ID) to add an IP to.
         :param address: The IP address to remove.
         :returns: An instance of novaclient.base.TupleWithMeta
         """
+        warnings.warn(ADD_REMOVE_FIXED_FLOATING_DEPRECATION_WARNING %
+                      'removeFixedIP', DeprecationWarning)
         return self._action('removeFixedIp', server, {'address': address})
 
+    @api_versions.wraps('2.0', '2.43')
     def add_floating_ip(self, server, address, fixed_address=None):
         """
-        Add a floating IP to an instance
+        DEPRECATED Add a floating IP to an instance
 
         :param server: The :class:`Server` (or its ID) to add an IP to.
         :param address: The FloatingIP or string floating address to add.
@@ -866,7 +927,8 @@ class ServerManager(base.BootingManagerWithFind):
                               associated with (optional)
         :returns: An instance of novaclient.base.TupleWithMeta
         """
-
+        warnings.warn(ADD_REMOVE_FIXED_FLOATING_DEPRECATION_WARNING %
+                      'addFloatingIP', DeprecationWarning)
         address = address.ip if hasattr(address, 'ip') else address
         if fixed_address:
             if hasattr(fixed_address, 'ip'):
@@ -877,19 +939,20 @@ class ServerManager(base.BootingManagerWithFind):
         else:
             return self._action('addFloatingIp', server, {'address': address})
 
+    @api_versions.wraps('2.0', '2.43')
     def remove_floating_ip(self, server, address):
         """
-        Remove a floating IP address.
+        DEPRECATED Remove a floating IP address.
 
         :param server: The :class:`Server` (or its ID) to remove an IP from.
         :param address: The FloatingIP or string floating address to remove.
         :returns: An instance of novaclient.base.TupleWithMeta
         """
-
+        warnings.warn(ADD_REMOVE_FIXED_FLOATING_DEPRECATION_WARNING %
+                      'removeFloatingIP', DeprecationWarning)
         address = address.ip if hasattr(address, 'ip') else address
         return self._action('removeFloatingIp', server, {'address': address})
 
-    @api_versions.wraps('2.0', '2.5')
     def get_vnc_console(self, server, console_type):
         """
         Get a vnc console for an instance
@@ -899,9 +962,8 @@ class ServerManager(base.BootingManagerWithFind):
         :returns: An instance of novaclient.base.DictWithMeta
         """
 
-        return self._action('os-getVNCConsole', server, {'type': console_type})
+        return self.get_console_url(server, console_type)
 
-    @api_versions.wraps('2.0', '2.5')
     def get_spice_console(self, server, console_type):
         """
         Get a spice console for an instance
@@ -911,10 +973,8 @@ class ServerManager(base.BootingManagerWithFind):
         :returns: An instance of novaclient.base.DictWithMeta
         """
 
-        return self._action('os-getSPICEConsole', server,
-                            {'type': console_type})
+        return self.get_console_url(server, console_type)
 
-    @api_versions.wraps('2.0', '2.5')
     def get_rdp_console(self, server, console_type):
         """
         Get a rdp console for an instance
@@ -924,73 +984,40 @@ class ServerManager(base.BootingManagerWithFind):
         :returns: An instance of novaclient.base.DictWithMeta
         """
 
-        return self._action('os-getRDPConsole', server,
-                            {'type': console_type})
+        return self.get_console_url(server, console_type)
+
+    def get_serial_console(self, server, console_type):
+        """
+        Get a serial console for an instance
+
+        :param server: The :class:`Server` (or its ID) to get console for.
+        :param console_type: Type of serial console to get ('serial')
+        :returns: An instance of novaclient.base.DictWithMeta
+        """
+
+        return self.get_console_url(server, console_type)
+
+    def _get_protocol(self, console_type):
+        protocol = CONSOLE_TYPE_PROTOCOL_MAPPING.get(console_type)
+        if not protocol:
+            raise exceptions.UnsupportedConsoleType(console_type)
+
+        return protocol
 
     @api_versions.wraps('2.0', '2.5')
-    def get_serial_console(self, server, console_type):
+    def get_console_url(self, server, console_type):
         """
-        Get a serial console for an instance
+        Retrieve a console url of a server.
 
-        :param server: The :class:`Server` (or its ID) to get console for.
-        :param console_type: Type of serial console to get ('serial')
-        :returns: An instance of novaclient.base.DictWithMeta
+        :param server: server to get console url for
+        :param console_type: type can be novnc, xvpvnc, spice-html5,
+                             rdp-html5 and serial.
         """
 
-        return self._action('os-getSerialConsole', server,
-                            {'type': console_type})
-
-    @api_versions.wraps('2.6')
-    def get_vnc_console(self, server, console_type):
-        """
-        Get a vnc console for an instance
-
-        :param server: The :class:`Server` (or its ID) to get console for.
-        :param console_type: Type of vnc console to get ('novnc' or 'xvpvnc')
-        :returns: An instance of novaclient.base.DictWithMeta
-        """
-
-        return self._console(server,
-                             {'protocol': 'vnc', 'type': console_type})
-
-    @api_versions.wraps('2.6')
-    def get_spice_console(self, server, console_type):
-        """
-        Get a spice console for an instance
-
-        :param server: The :class:`Server` (or its ID) to get console for.
-        :param console_type: Type of spice console to get ('spice-html5')
-        :returns: An instance of novaclient.base.DictWithMeta
-        """
-
-        return self._console(server,
-                             {'protocol': 'spice', 'type': console_type})
-
-    @api_versions.wraps('2.6')
-    def get_rdp_console(self, server, console_type):
-        """
-        Get a rdp console for an instance
-
-        :param server: The :class:`Server` (or its ID) to get console for.
-        :param console_type: Type of rdp console to get ('rdp-html5')
-        :returns: An instance of novaclient.base.DictWithMeta
-        """
-
-        return self._console(server,
-                             {'protocol': 'rdp', 'type': console_type})
-
-    @api_versions.wraps('2.6')
-    def get_serial_console(self, server, console_type):
-        """
-        Get a serial console for an instance
-
-        :param server: The :class:`Server` (or its ID) to get console for.
-        :param console_type: Type of serial console to get ('serial')
-        :returns: An instance of novaclient.base.DictWithMeta
-        """
-
-        return self._console(server,
-                             {'protocol': 'serial', 'type': console_type})
+        action = CONSOLE_TYPE_ACTION_MAPPING.get(console_type)
+        if not action:
+            raise exceptions.UnsupportedConsoleType(console_type)
+        return self._action(action, server, {'type': console_type})
 
     @api_versions.wraps('2.8')
     def get_mks_console(self, server):
@@ -1001,8 +1028,30 @@ class ServerManager(base.BootingManagerWithFind):
         :returns: An instance of novaclient.base.DictWithMeta
         """
 
-        return self._console(server,
-                             {'protocol': 'mks', 'type': 'webmks'})
+        return self.get_console_url(server, 'webmks')
+
+    @api_versions.wraps('2.6')
+    def get_console_url(self, server, console_type):
+        """
+        Retrieve a console url of a server.
+
+        :param server: server to get console url for
+        :param console_type: type can be novnc/xvpvnc for protocol vnc;
+                             spice-html5 for protocol spice; rdp-html5 for
+                             protocol rdp; serial for protocol serial.
+                             webmks for protocol mks (since version 2.8).
+        """
+
+        if self.api_version < api_versions.APIVersion('2.8'):
+            if console_type == 'webmks':
+                raise exceptions.UnsupportedConsoleType(console_type)
+
+        protocol = self._get_protocol(console_type)
+        body = {'remote_console': {'protocol': protocol,
+                                   'type': console_type}}
+        url = '/servers/%s/remote-consoles' % base.getid(server)
+        resp, body = self.api.client.post(url, body=body)
+        return self.convert_into_with_meta(body, resp)
 
     def get_password(self, server, private_key=None):
         """
@@ -1219,6 +1268,14 @@ class ServerManager(base.BootingManagerWithFind):
                                          base.getid(server))
         return base.TupleWithMeta((resp, body), resp)
 
+    def _validate_create_nics(self, nics):
+        # nics are required with microversion 2.37+ and can be a string or list
+        if self.api_version > api_versions.APIVersion('2.36'):
+            if not nics:
+                raise ValueError('nics are required after microversion 2.36')
+        elif nics and not isinstance(nics, list):
+            raise ValueError('nics must be a list')
+
     def create(self, name, image, flavor, meta=None, files=None,
                reservation_id=None, min_count=None,
                max_count=None, security_groups=None, userdata=None,
@@ -1259,9 +1316,17 @@ class ServerManager(base.BootingManagerWithFind):
                       device mappings for this server.
         :param block_device_mapping_v2: (optional extension) A dict of block
                       device mappings for this server.
-        :param nics:  (optional extension) an ordered list of nics to be
-                      added to this server, with information about
-                      connected networks, fixed IPs, port etc.
+        :param nics:  An ordered list of nics (dicts) to be added to this
+                      server, with information about connected networks,
+                      fixed IPs, port etc.
+                      Beginning in microversion 2.37 this field is required and
+                      also supports a single string value of 'auto' or 'none'.
+                      The 'auto' value means the Compute service will
+                      automatically allocate a network for the project if one
+                      is not available. This is the same behavior as not
+                      passing anything for nics before microversion 2.37. The
+                      'none' value tells the Compute service to not allocate
+                      any networking for the server.
         :param scheduler_hints: (optional extension) arbitrary key-value pairs
                             specified by the client to help boot an instance
         :param config_drive: (optional extension) value for config drive
@@ -1275,6 +1340,8 @@ class ServerManager(base.BootingManagerWithFind):
         :param access_ip_v6: (optional extension) add alternative access ip v6
         :param description: optional description of the server (allowed since
                             microversion 2.19)
+        :param tags: A list of arbitrary strings to be added to the
+                     server as tags (allowed since microversion 2.52)
         """
         if not min_count:
             min_count = 1
@@ -1289,6 +1356,8 @@ class ServerManager(base.BootingManagerWithFind):
         if "description" in kwargs and self.api_version < descr_microversion:
             raise exceptions.UnsupportedAttribute("description", "2.19")
 
+        self._validate_create_nics(nics)
+
         tags_microversion = api_versions.APIVersion("2.32")
         if self.api_version < tags_microversion:
             if nics:
@@ -1305,6 +1374,10 @@ class ServerManager(base.BootingManagerWithFind):
                                          "unsupported before microversion "
                                          "2.32")
 
+        boot_tags_microversion = api_versions.APIVersion("2.52")
+        if "tags" in kwargs and self.api_version < boot_tags_microversion:
+            raise exceptions.UnsupportedAttribute("tags", "2.52")
+
         boot_kwargs = dict(
             meta=meta, files=files, userdata=userdata,
             reservation_id=reservation_id, min_count=min_count,
@@ -1523,8 +1596,13 @@ class ServerManager(base.BootingManagerWithFind):
         body = {'name': image_name, 'metadata': metadata or {}}
         resp, body = self._action_return_resp_and_body('createImage', server,
                                                        body)
-        location = resp.headers['location']
-        image_uuid = location.split('/')[-1]
+        # The 2.45 microversion returns the image_id in the response body,
+        # not as a location header.
+        if self.api_version >= api_versions.APIVersion('2.45'):
+            image_uuid = body['image_id']
+        else:
+            location = resp.headers['location']
+            image_uuid = location.split('/')[-1]
         return base.StrWithMeta(image_uuid, resp)
 
     def backup(self, server, backup_name, backup_type, rotation):
@@ -1536,7 +1614,8 @@ class ServerManager(base.BootingManagerWithFind):
         :param backup_type: The backup type, like 'daily' or 'weekly'
         :param rotation: Int parameter representing how many backups to
                         keep around.
-        :returns: An instance of novaclient.base.TupleWithMeta
+        :returns: An instance of novaclient.base.TupleWithMeta if the request
+            microversion is < 2.45, otherwise novaclient.base.DictWithMeta.
         """
         body = {'name': backup_name,
                 'backup_type': backup_type,
@@ -1545,9 +1624,9 @@ class ServerManager(base.BootingManagerWithFind):
 
     def set_meta(self, server, metadata):
         """
-        Set a servers metadata
+        Set a server's metadata
         :param server: The :class:`Server` to add metadata to
-        :param metadata: A dict of metadata to add to the server
+        :param metadata: A dict of metadata to be added to the server
         """
         body = {'metadata': metadata}
         return self._create("/servers/%s/metadata" % base.getid(server),
@@ -1694,7 +1773,7 @@ class ServerManager(base.BootingManagerWithFind):
         """
         return self._list('/servers/%s/os-security-groups' %
                           base.getid(server), 'security_groups',
-                          security_groups.SecurityGroup)
+                          SecurityGroup)
 
     @api_versions.wraps("2.0", "2.13")
     def evacuate(self, server, host=None, on_shared_storage=True,
@@ -1778,6 +1857,7 @@ class ServerManager(base.BootingManagerWithFind):
         return self._list('/servers/%s/os-interface' % base.getid(server),
                           'interfaceAttachments', obj_class=NetworkInterface)
 
+    @api_versions.wraps("2.0", "2.48")
     def interface_attach(self, server, port_id, net_id, fixed_ip):
         """
         Attach a network_interface to an instance.
@@ -1798,6 +1878,35 @@ class ServerManager(base.BootingManagerWithFind):
         return self._create('/servers/%s/os-interface' % base.getid(server),
                             body, 'interfaceAttachment')
 
+    @api_versions.wraps("2.49")
+    def interface_attach(self, server, port_id, net_id, fixed_ip, tag=None):
+        """
+        Attach a network_interface to an instance.
+
+        :param server: The :class:`Server` (or its ID) to attach to.
+        :param port_id: The port to attach.
+                        The port_id and net_id parameters are mutually
+                        exclusive.
+        :param net_id: The ID of the network to attach.
+        :param fixed_ip: The fixed IP addresses. If the fixed_ip is specified,
+                         the net_id has to be specified at the same time.
+        :param tag: The tag.
+        """
+
+        body = {'interfaceAttachment': {}}
+        if port_id:
+            body['interfaceAttachment']['port_id'] = port_id
+        if net_id:
+            body['interfaceAttachment']['net_id'] = net_id
+        if fixed_ip:
+            body['interfaceAttachment']['fixed_ips'] = [
+                {'ip_address': fixed_ip}]
+        if tag:
+            body['interfaceAttachment']['tag'] = tag
+
+        return self._create('/servers/%s/os-interface' % base.getid(server),
+                            body, 'interfaceAttachment')
+
     def interface_detach(self, server, port_id):
         """
         Detach a network_interface from an instance.
@@ -1832,15 +1941,6 @@ class ServerManager(base.BootingManagerWithFind):
         url = '/servers/%s/action' % base.getid(server)
         return self.api.client.post(url, body=body)
 
-    def _console(self, server, info=None, **kwargs):
-        """
-        Retrieve a console of a particular protocol -- vnc/spice/rdp/serial
-        """
-        body = {'remote_console': info}
-        url = '/servers/%s/remote-consoles' % base.getid(server)
-        resp, body = self.api.client.post(url, body=body)
-        return self.convert_into_with_meta(body, resp)
-
     @api_versions.wraps('2.26')
     def tag_list(self, server):
         """
